@@ -17,10 +17,11 @@
 //! Common types.
 
 use crate::{
-    cbor::{reader::DecoderError, values::Value, writer::EncoderError},
+    cbor,
+    cbor::value::Value,
     iana,
     iana::{EnumI64, WithPrivateRange},
-    util::{cbor_type_error, AsCborValue},
+    util::{cbor_type_error, AsCborValue, ValueTryAs},
 };
 use alloc::{boxed::Box, string::String, vec::Vec};
 use core::{cmp::Ordering, convert::TryInto};
@@ -28,38 +29,61 @@ use core::{cmp::Ordering, convert::TryInto};
 #[cfg(test)]
 mod tests;
 
+/// Marker structure indicating that the EOF was encountered when reading CBOR data.
+#[derive(Debug)]
+pub struct EndOfFile;
+
 /// Error type for failures in encoding or decoding COSE types.
 pub enum CoseError {
     /// CBOR decoding failure.
-    DecodeFailed(DecoderError),
+    DecodeFailed(cbor::de::Error<EndOfFile>),
+    /// Duplicate map key detected.
+    DuplicateMapKey,
     /// CBOR encoding failure.
-    EncodeFailed(EncoderError),
-    /// Unexpected CBOR type encountered (got, want).
-    UnexpectedType(&'static str, &'static str),
+    EncodeFailed,
+    /// CBOR input had extra data.
+    ExtraneousData,
+    /// Integer value on the wire is outside the range of integers representable in this crate.
+    /// See <https://crates.io/crates/coset/#integer-ranges>.
+    OutOfRangeIntegerValue,
+    /// Unexpected CBOR item encountered (got, want).
+    UnexpectedItem(&'static str, &'static str),
     /// Unrecognized value in IANA-controlled range (with no private range).
     UnregisteredIanaValue,
     /// Unrecognized value in neither IANA-controlled range nor private range.
     UnregisteredIanaNonPrivateValue,
 }
 
-impl core::convert::From<DecoderError> for CoseError {
-    fn from(e: DecoderError) -> Self {
+/// Crate-specific Result type
+pub type Result<T, E = CoseError> = core::result::Result<T, E>;
+
+impl core::convert::From<cbor::de::Error<EndOfFile>> for CoseError {
+    fn from(e: cbor::de::Error<EndOfFile>) -> Self {
         CoseError::DecodeFailed(e)
     }
 }
 
-impl core::convert::From<EncoderError> for CoseError {
-    fn from(e: EncoderError) -> Self {
-        CoseError::EncodeFailed(e)
+impl<T> core::convert::From<cbor::ser::Error<T>> for CoseError {
+    fn from(_e: cbor::ser::Error<T>) -> Self {
+        CoseError::EncodeFailed
+    }
+}
+
+impl core::convert::From<core::num::TryFromIntError> for CoseError {
+    fn from(_: core::num::TryFromIntError) -> Self {
+        CoseError::OutOfRangeIntegerValue
     }
 }
 
 impl core::fmt::Debug for CoseError {
     fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
         match self {
-            CoseError::DecodeFailed(de) => write!(f, "decode CBOR failure: {:?}", de),
-            CoseError::EncodeFailed(ee) => write!(f, "encode CBOR failure: {:?}", ee),
-            CoseError::UnexpectedType(got, want) => write!(f, "got {}, expected {}", got, want),
+            CoseError::DecodeFailed(e) => write!(f, "decode CBOR failure: {}", e),
+            CoseError::DuplicateMapKey => write!(f, "duplicate map key"),
+            CoseError::EncodeFailed => write!(f, "encode CBOR failure"),
+            CoseError::ExtraneousData => write!(f, "extraneous data in CBOR input"),
+            CoseError::OutOfRangeIntegerValue => write!(f, "out of range integer value"),
+            CoseError::UnexpectedItem(got, want) => write!(f, "got {}, expected {}", got, want),
             CoseError::UnregisteredIanaValue => write!(f, "expected recognized IANA value"),
             CoseError::UnregisteredIanaNonPrivateValue => {
                 write!(f, "expected value in IANA or private use range")
@@ -68,18 +92,57 @@ impl core::fmt::Debug for CoseError {
     }
 }
 
+/// Newtype wrapper around a byte slice to allow left-over data to be detected.
+struct MeasuringReader<'a>(&'a [u8]);
+
+impl<'a> MeasuringReader<'a> {
+    fn new(buf: &'a [u8]) -> MeasuringReader<'a> {
+        MeasuringReader(buf)
+    }
+
+    fn is_empty(&self) -> bool {
+        self.0.is_empty()
+    }
+}
+
+impl<'a> ciborium_io::Read for &mut MeasuringReader<'a> {
+    type Error = EndOfFile;
+
+    fn read_exact(&mut self, data: &mut [u8]) -> Result<(), Self::Error> {
+        if data.len() > self.0.len() {
+            return Err(EndOfFile);
+        }
+
+        let (prefix, suffix) = self.0.split_at(data.len());
+        data.copy_from_slice(prefix);
+        self.0 = suffix;
+        Ok(())
+    }
+}
+
+/// Read a CBOR [`Value`] from a byte slice, failing if any extra data remains after the `Value` has
+/// been read.
+fn read_to_value(slice: &[u8]) -> Result<Value> {
+    let mut mr = MeasuringReader::new(slice);
+    let value = cbor::de::from_reader(&mut mr)?;
+    if mr.is_empty() {
+        Ok(value)
+    } else {
+        Err(CoseError::ExtraneousData)
+    }
+}
+
 /// Extension trait that adds serialization/deserialization methods.
 pub trait CborSerializable: AsCborValue {
     /// Create an object instance from serialized CBOR data in a slice.
-    fn from_slice(slice: &[u8]) -> Result<Self, CoseError> {
-        let value = sk_cbor::reader::read(slice).map_err(CoseError::DecodeFailed)?;
-        Self::from_cbor_value(value)
+    fn from_slice(slice: &[u8]) -> Result<Self> {
+        Self::from_cbor_value(read_to_value(slice)?)
     }
 
     /// Serialize this object to a vector, consuming it along the way.
-    fn to_vec(self) -> Result<Vec<u8>, CoseError> {
+    fn to_vec(self) -> Result<Vec<u8>> {
         let mut data = Vec::new();
-        sk_cbor::writer::write(self.to_cbor_value()?, &mut data)?;
+        cbor::ser::into_writer(&self.to_cbor_value()?, &mut data)?;
         Ok(data)
     }
 }
@@ -91,19 +154,20 @@ pub trait TaggedCborSerializable: AsCborValue {
 
     /// Create an object instance from serialized CBOR data in a slice, expecting an initial
     /// tag value.
-    fn from_tagged_slice(slice: &[u8]) -> Result<Self, CoseError> {
-        match sk_cbor::reader::read(slice)? {
-            Value::Tag(t, v) if t == Self::TAG => Self::from_cbor_value(*v),
-            v => cbor_type_error(&v, "tag"),
+    fn from_tagged_slice(slice: &[u8]) -> Result<Self> {
+        let (t, v) = read_to_value(slice)?.try_as_tag()?;
+        if t != Self::TAG {
+            return Err(CoseError::UnexpectedItem("tag", "other tag"));
         }
+        Self::from_cbor_value(*v)
     }
 
     /// Serialize this object to a vector, including initial tag, consuming the object along the
     /// way.
-    fn to_tagged_vec(self) -> Result<Vec<u8>, CoseError> {
+    fn to_tagged_vec(self) -> Result<Vec<u8>> {
         let mut data = Vec::new();
-        sk_cbor::writer::write(
-            Value::Tag(Self::TAG, Box::new(self.to_cbor_value()?)),
+        cbor::ser::into_writer(
+            &Value::Tag(Self::TAG, Box::new(self.to_cbor_value()?)),
             &mut data,
         )?;
         Ok(data)
@@ -162,22 +226,17 @@ impl PartialOrd for Label {
 }
 
 impl AsCborValue for Label {
-    fn from_cbor_value(value: Value) -> Result<Self, CoseError> {
+    fn from_cbor_value(value: Value) -> Result<Self> {
         match value {
-            Value::Unsigned(u) => Ok(Label::Int(
-                u.try_into()
-                    .map_err(|_e| CoseError::UnexpectedType("u64", "u63"))?,
-            )),
-            Value::Negative(i) => Ok(Label::Int(i)),
-            Value::TextString(t) => Ok(Label::Text(t)),
+            Value::Integer(i) => Ok(Label::Int(i.try_into()?)),
+            Value::Text(t) => Ok(Label::Text(t)),
             v => cbor_type_error(&v, "int/tstr"),
         }
     }
-    fn to_cbor_value(self) -> Result<Value, CoseError> {
+    fn to_cbor_value(self) -> Result<Value> {
         Ok(match self {
-            Label::Int(i) if i < 0 => Value::Negative(i),
-            Label::Int(i) => Value::Unsigned(i as u64), // safe: i64 value that is >=0 fits in u64
-            Label::Text(t) => Value::TextString(t),
+            Label::Int(i) => Value::from(i),
+            Label::Text(t) => Value::Text(t),
         })
     }
 }
@@ -215,40 +274,24 @@ impl<T: EnumI64> PartialOrd for RegisteredLabel<T> {
 }
 
 impl<T: EnumI64> AsCborValue for RegisteredLabel<T> {
-    fn from_cbor_value(value: Value) -> Result<Self, CoseError> {
+    fn from_cbor_value(value: Value) -> Result<Self> {
         match value {
-            Value::Unsigned(u) => {
-                let i: i64 = u
-                    .try_into()
-                    .map_err(|_e| CoseError::UnexpectedType("u64", "u63"))?;
-                if let Some(a) = T::from_i64(i) {
+            Value::Integer(i) => {
+                if let Some(a) = T::from_i64(i.try_into()?) {
                     Ok(RegisteredLabel::Assigned(a))
                 } else {
                     Err(CoseError::UnregisteredIanaValue)
                 }
             }
-            Value::Negative(i) => {
-                if let Some(a) = T::from_i64(i) {
-                    Ok(RegisteredLabel::Assigned(a))
-                } else {
-                    Err(CoseError::UnregisteredIanaValue)
-                }
-            }
-            Value::TextString(t) => Ok(RegisteredLabel::Text(t)),
+            Value::Text(t) => Ok(RegisteredLabel::Text(t)),
             v => cbor_type_error(&v, "int/tstr"),
         }
     }
-    fn to_cbor_value(self) -> Result<Value, CoseError> {
+
+    fn to_cbor_value(self) -> Result<Value> {
         Ok(match self {
-            RegisteredLabel::Assigned(e) => {
-                let e64 = e.to_i64();
-                if e64 >= 0 {
-                    Value::Unsigned(e64 as u64) // safe: i64 value that is >= 0 fits in u64
-                } else {
-                    Value::Negative(e64)
-                }
-            }
-            RegisteredLabel::Text(t) => Value::TextString(t),
+            RegisteredLabel::Assigned(e) => Value::from(e.to_i64()),
+            RegisteredLabel::Text(t) => Value::Text(t),
         })
     }
 }
@@ -290,12 +333,10 @@ impl<T: EnumI64 + WithPrivateRange> PartialOrd for RegisteredLabelWithPrivate<T>
 }
 
 impl<T: EnumI64 + WithPrivateRange> AsCborValue for RegisteredLabelWithPrivate<T> {
-    fn from_cbor_value(value: Value) -> Result<Self, CoseError> {
+    fn from_cbor_value(value: Value) -> Result<Self> {
         match value {
-            Value::Unsigned(u) => {
-                let i = u
-                    .try_into()
-                    .map_err(|_e| CoseError::UnexpectedType("u64", "u63"))?;
+            Value::Integer(i) => {
+                let i = i.try_into()?;
                 if let Some(a) = T::from_i64(i) {
                     Ok(RegisteredLabelWithPrivate::Assigned(a))
                 } else if T::is_private(i) {
@@ -304,37 +345,15 @@ impl<T: EnumI64 + WithPrivateRange> AsCborValue for RegisteredLabelWithPrivate<T
                     Err(CoseError::UnregisteredIanaNonPrivateValue)
                 }
             }
-            Value::Negative(i) => {
-                if let Some(a) = T::from_i64(i) {
-                    Ok(RegisteredLabelWithPrivate::Assigned(a))
-                } else if T::is_private(i) {
-                    Ok(RegisteredLabelWithPrivate::PrivateUse(i))
-                } else {
-                    Err(CoseError::UnregisteredIanaNonPrivateValue)
-                }
-            }
-            Value::TextString(t) => Ok(RegisteredLabelWithPrivate::Text(t)),
+            Value::Text(t) => Ok(RegisteredLabelWithPrivate::Text(t)),
             v => cbor_type_error(&v, "int/tstr"),
         }
     }
-    fn to_cbor_value(self) -> Result<Value, CoseError> {
+    fn to_cbor_value(self) -> Result<Value> {
         Ok(match self {
-            RegisteredLabelWithPrivate::PrivateUse(i) => {
-                if i >= 0 {
-                    Value::Unsigned(i as u64) // safe: i64 value that is >=0 fits in u64
-                } else {
-                    Value::Negative(i)
-                }
-            }
-            RegisteredLabelWithPrivate::Assigned(i) => {
-                let i = i.to_i64();
-                if i >= 0 {
-                    Value::Unsigned(i as u64) // safe: i64 value that is >=0 fits in u64
-                } else {
-                    Value::Negative(i)
-                }
-            }
-            RegisteredLabelWithPrivate::Text(t) => Value::TextString(t),
+            RegisteredLabelWithPrivate::PrivateUse(i) => Value::from(i),
+            RegisteredLabelWithPrivate::Assigned(i) => Value::from(i.to_i64()),
+            RegisteredLabelWithPrivate::Text(t) => Value::Text(t),
         })
     }
 }
