@@ -201,7 +201,8 @@ fn test_context_encode() {
         let got = key.clone().to_vec().unwrap();
         assert_eq!(*key_data, hex::encode(&got), "case {}", i);
 
-        let got = CoseKdfContext::from_slice(&got).unwrap();
+        let mut got = CoseKdfContext::from_slice(&got).unwrap();
+        got.supp_pub_info.protected.original_data = None;
         assert_eq!(*key, got);
     }
 }
@@ -235,7 +236,7 @@ fn test_context_decode_fail() {
                 "83", "f6f6f6", // 3-tuple: [nil, nil, nil]
                 "83", "f6f6f6", // 3-tuple: [nil, nil, nil]
             ),
-            "IncompleteCborData",
+            "decode CBOR failure: Io(EndOfFile",
         ),
         (
             concat!(
@@ -335,7 +336,7 @@ fn test_context_decode_fail() {
                 "83", "f6f6f6", // 3-tuple: [nil, nil, nil]
                 "82", "4040", // 2-tuple: [0-bstr, 0-bstr]
             ),
-            "expected uint",
+            "expected int",
         ),
         (
             concat!(
@@ -345,7 +346,7 @@ fn test_context_decode_fail() {
                 "83", "f6f6f6", // 3-tuple: [nil, nil, nil]
                 "82", "0060", // 2-tuple: [0, 0-tstr]
             ),
-            "expected bstr encoded map",
+            "expected bstr",
         ),
         (
             concat!(
@@ -368,6 +369,19 @@ fn test_context_decode_fail() {
             ),
             "expected bstr",
         ),
+        (
+            concat!(
+                "84", // 4-tuple
+                "01", // int : AES-128-GCM
+                "83", // 3-tuple: [0-bstr, out-of-range int, nil]
+                "401b8000000000000000f6",
+                "83", // 3-tuple: [nil, nil, nil]
+                "f6f6f6",
+                "82", // 2-tuple: [0, 0-bstr]
+                "0040",
+            ),
+            "out of range integer value",
+        ),
     ];
     for (context_data, err_msg) in tests.iter() {
         let data = hex::decode(context_data).unwrap();
