@@ -17,11 +17,11 @@
 //! COSE_Key functionality.
 
 use crate::{
-    cbor::values::{SimpleValue, Value},
+    cbor::value::Value,
     iana,
     iana::EnumI64,
-    util::{cbor_type_error, AsCborValue},
-    Algorithm, CoseError, Label,
+    util::{to_cbor_array, AsCborValue, ValueTryAs},
+    Algorithm, CoseError, Label, Result,
 };
 use alloc::{collections::BTreeSet, vec, vec::Vec};
 
@@ -41,30 +41,20 @@ impl Default for KeyType {
 pub type KeyOperation = crate::RegisteredLabel<iana::KeyOperation>;
 
 /// A collection of [`CoseKey`] objects.
-#[derive(Clone, Debug, Default, Eq, PartialEq)]
+#[derive(Clone, Debug, Default, PartialEq)]
 pub struct CoseKeySet(pub Vec<CoseKey>);
 
 impl crate::CborSerializable for CoseKeySet {}
 
 impl AsCborValue for CoseKeySet {
-    fn from_cbor_value(value: Value) -> Result<Self, CoseError> {
-        let a = match value {
-            Value::Array(a) => a,
-            v => return cbor_type_error(&v, "array"),
-        };
-        let mut keys = Vec::new();
-        for v in a {
-            keys.push(CoseKey::from_cbor_value(v)?);
-        }
-        Ok(Self(keys))
+    fn from_cbor_value(value: Value) -> Result<Self> {
+        Ok(Self(
+            value.try_as_array_then_convert(CoseKey::from_cbor_value)?,
+        ))
     }
 
-    fn to_cbor_value(self) -> Result<Value, CoseError> {
-        let mut arr = Vec::new();
-        for k in self.0 {
-            arr.push(k.to_cbor_value()?);
-        }
-        Ok(Value::Array(arr))
+    fn to_cbor_value(self) -> Result<Value> {
+        to_cbor_array(self.0)
     }
 }
 
@@ -80,7 +70,7 @@ impl AsCborValue for CoseKeySet {
 ///      * label => values
 ///  }
 ///  ```
-#[derive(Clone, Debug, Default, Eq, PartialEq)]
+#[derive(Clone, Debug, Default, PartialEq)]
 pub struct CoseKey {
     /// Key type identification.
     pub kty: KeyType,
@@ -99,75 +89,59 @@ pub struct CoseKey {
 
 impl crate::CborSerializable for CoseKey {}
 
-const KTY: Value = Value::Unsigned(iana::KeyParameter::Kty as u64);
-const KID: Value = Value::Unsigned(iana::KeyParameter::Kid as u64);
-const ALG: Value = Value::Unsigned(iana::KeyParameter::Alg as u64);
-const KEY_OPS: Value = Value::Unsigned(iana::KeyParameter::KeyOps as u64);
-const BASE_IV: Value = Value::Unsigned(iana::KeyParameter::BaseIv as u64);
+const KTY: Label = Label::Int(iana::KeyParameter::Kty as i64);
+const KID: Label = Label::Int(iana::KeyParameter::Kid as i64);
+const ALG: Label = Label::Int(iana::KeyParameter::Alg as i64);
+const KEY_OPS: Label = Label::Int(iana::KeyParameter::KeyOps as i64);
+const BASE_IV: Label = Label::Int(iana::KeyParameter::BaseIv as i64);
 
 impl AsCborValue for CoseKey {
-    fn from_cbor_value(value: Value) -> Result<Self, CoseError> {
-        let m = match value {
-            Value::Map(m) => m,
-            v => return cbor_type_error(&v, "map"),
-        };
-
+    fn from_cbor_value(value: Value) -> Result<Self> {
+        let m = value.try_as_map()?;
         let mut key = Self::default();
-        for (label, value) in m.into_iter() {
+        let mut seen = BTreeSet::new();
+        for (l, value) in m.into_iter() {
+            // The `ciborium` CBOR library does not police duplicate map keys.
+            // RFC 8152 section 14 requires that COSE does police duplicates, so do it here.
+            let label = Label::from_cbor_value(l)?;
+            if seen.contains(&label) {
+                return Err(CoseError::DuplicateMapKey);
+            }
+            seen.insert(label.clone());
             match label {
-                x if x == KTY => key.kty = KeyType::from_cbor_value(value)?,
+                KTY => key.kty = KeyType::from_cbor_value(value)?,
 
-                x if x == KID => match value {
-                    Value::ByteString(v) => {
-                        if v.is_empty() {
-                            return Err(CoseError::UnexpectedType("empty bstr", "non-empty bstr"));
-                        }
-                        key.key_id = v;
-                    }
-                    v => return cbor_type_error(&v, "bstr value"),
-                },
+                KID => {
+                    key.key_id = value.try_as_nonempty_bytes()?;
+                }
 
-                x if x == ALG => key.alg = Some(Algorithm::from_cbor_value(value)?),
+                ALG => key.alg = Some(Algorithm::from_cbor_value(value)?),
 
-                x if x == KEY_OPS => match value {
-                    Value::Array(key_ops) => {
-                        for key_op in key_ops.into_iter() {
-                            if !key.key_ops.insert(KeyOperation::from_cbor_value(key_op)?) {
-                                return Err(CoseError::UnexpectedType(
-                                    "repeated array entry",
-                                    "unique array label",
-                                ));
-                            }
-                        }
-                        if key.key_ops.is_empty() {
-                            return Err(CoseError::UnexpectedType(
-                                "empty array",
-                                "non-empty array",
+                KEY_OPS => {
+                    let key_ops = value.try_as_array()?;
+                    for key_op in key_ops.into_iter() {
+                        if !key.key_ops.insert(KeyOperation::from_cbor_value(key_op)?) {
+                            return Err(CoseError::UnexpectedItem(
+                                "repeated array entry",
+                                "unique array label",
                             ));
                         }
                     }
-                    v => return cbor_type_error(&v, "array value"),
-                },
-
-                x if x == BASE_IV => match value {
-                    Value::ByteString(v) => {
-                        if v.is_empty() {
-                            return Err(CoseError::UnexpectedType("empty bstr", "non-empty bstr"));
-                        }
-                        key.base_iv = v;
+                    if key.key_ops.is_empty() {
+                        return Err(CoseError::UnexpectedItem("empty array", "non-empty array"));
                     }
-                    v => return cbor_type_error(&v, "bstr value"),
-                },
-
-                l => {
-                    let label = Label::from_cbor_value(l)?;
-                    key.params.push((label, value));
                 }
+
+                BASE_IV => {
+                    key.base_iv = value.try_as_nonempty_bytes()?;
+                }
+
+                label => key.params.push((label, value)),
             }
         }
         // Check that key type has been set.
         if key.kty == KeyType::Assigned(iana::KeyType::Reserved) {
-            return Err(CoseError::UnexpectedType(
+            return Err(CoseError::UnexpectedItem(
                 "no kty label",
                 "mandatory kty label",
             ));
@@ -176,25 +150,26 @@ impl AsCborValue for CoseKey {
         Ok(key)
     }
 
-    fn to_cbor_value(self) -> Result<Value, CoseError> {
-        let mut map: Vec<(Value, Value)> = vec![(KTY, self.kty.to_cbor_value()?)];
+    fn to_cbor_value(self) -> Result<Value> {
+        let mut map: Vec<(Value, Value)> = vec![(KTY.to_cbor_value()?, self.kty.to_cbor_value()?)];
         if !self.key_id.is_empty() {
-            map.push((KID, Value::ByteString(self.key_id)));
+            map.push((KID.to_cbor_value()?, Value::Bytes(self.key_id)));
         }
         if let Some(alg) = self.alg {
-            map.push((ALG, alg.to_cbor_value()?));
+            map.push((ALG.to_cbor_value()?, alg.to_cbor_value()?));
         }
         if !self.key_ops.is_empty() {
-            let mut arr = Vec::new();
-            for op in self.key_ops {
-                arr.push(op.to_cbor_value()?);
-            }
-            map.push((KEY_OPS, Value::Array(arr)));
+            map.push((KEY_OPS.to_cbor_value()?, to_cbor_array(self.key_ops)?));
         }
         if !self.base_iv.is_empty() {
-            map.push((BASE_IV, Value::ByteString(self.base_iv)));
+            map.push((BASE_IV.to_cbor_value()?, Value::Bytes(self.base_iv)));
         }
+        let mut seen = BTreeSet::new();
         for (label, value) in self.params {
+            if seen.contains(&label) {
+                return Err(CoseError::DuplicateMapKey);
+            }
+            seen.insert(label.clone());
             map.push((label.to_cbor_value()?, value));
         }
         Ok(Value::Map(map))
@@ -217,16 +192,10 @@ impl CoseKeyBuilder {
             params: vec![
                 (
                     Label::Int(iana::Ec2KeyParameter::Crv as i64),
-                    Value::Unsigned(curve as u64),
-                ),
-                (
-                    Label::Int(iana::Ec2KeyParameter::X as i64),
-                    Value::ByteString(x),
-                ),
-                (
-                    Label::Int(iana::Ec2KeyParameter::Y as i64),
-                    Value::ByteString(y),
+                    Value::from(curve as u64),
                 ),
+                (Label::Int(iana::Ec2KeyParameter::X as i64), Value::Bytes(x)),
+                (Label::Int(iana::Ec2KeyParameter::Y as i64), Value::Bytes(y)),
             ],
             ..Default::default()
         })
@@ -240,18 +209,12 @@ impl CoseKeyBuilder {
             params: vec![
                 (
                     Label::Int(iana::Ec2KeyParameter::Crv as i64),
-                    Value::Unsigned(curve as u64),
-                ),
-                (
-                    Label::Int(iana::Ec2KeyParameter::X as i64),
-                    Value::ByteString(x),
+                    Value::from(curve as u64),
                 ),
+                (Label::Int(iana::Ec2KeyParameter::X as i64), Value::Bytes(x)),
                 (
                     Label::Int(iana::Ec2KeyParameter::Y as i64),
-                    match y_sign {
-                        true => Value::Simple(SimpleValue::TrueValue),
-                        false => Value::Simple(SimpleValue::FalseValue),
-                    },
+                    Value::Bool(y_sign),
                 ),
             ],
             ..Default::default()
@@ -267,10 +230,10 @@ impl CoseKeyBuilder {
         d: Vec<u8>,
     ) -> Self {
         let mut builder = Self::new_ec2_pub_key(curve, x, y);
-        builder.0.params.push((
-            Label::Int(iana::Ec2KeyParameter::D as i64),
-            Value::ByteString(d),
-        ));
+        builder
+            .0
+            .params
+            .push((Label::Int(iana::Ec2KeyParameter::D as i64), Value::Bytes(d)));
         builder
     }
 
@@ -280,19 +243,21 @@ impl CoseKeyBuilder {
             kty: KeyType::Assigned(iana::KeyType::Symmetric),
             params: vec![(
                 Label::Int(iana::SymmetricKeyParameter::K as i64),
-                Value::ByteString(k),
+                Value::Bytes(k),
             )],
             ..Default::default()
         })
     }
 
     /// Set the algorithm.
+    #[must_use]
     pub fn algorithm(mut self, alg: iana::Algorithm) -> Self {
         self.0.alg = Some(Algorithm::Assigned(alg));
         self
     }
 
     /// Add a key operation.
+    #[must_use]
     pub fn add_key_op(mut self, op: iana::KeyOperation) -> Self {
         self.0.key_ops.insert(KeyOperation::Assigned(op));
         self
@@ -304,6 +269,7 @@ impl CoseKeyBuilder {
     ///
     /// This function will panic if it used to set a parameter label from the [`iana::KeyParameter`]
     /// range.
+    #[must_use]
     pub fn param(mut self, label: i64, value: Value) -> Self {
         if iana::KeyParameter::from_i64(label).is_some() {
             panic!("param() method used to set KeyParameter"); // safe: invalid input
